@@ -1,11 +1,18 @@
-/*	This is pstree written by Fred Hucht (c) 1993-2000	*
+/*	This is pstree written by Fred Hucht (c) 1993-2002	*
  *	EMail: fred@thp.Uni-Duisburg.DE				*
  *	Feel free to copy and redistribute in terms of the	*
  * 	GNU public license. 					*
  *
- * $Id: pstree.c,v 2.16 2000-03-01 10:42:22+01 fred Exp fred $
+ * $Id: pstree.c,v 2.17 2001-12-17 13:18:02+01 fred Exp $
  *
  * $Log: pstree.c,v $
+ * Revision 2.17  2001-12-17 13:18:02+01  fred
+ * Changed ps call to something like ps -eo uid,pid,ppid,pgid,args under
+ * AIX and Linux, workaround for AIX 5L.
+ *
+ * Revision 2.17  2001-12-13 08:27:00+08  chris
+ * Added workaround for AIX Version >= 5
+ *
  * Revision 2.16  2000-03-01 10:42:22+01  fred
  * Added support for thread count (thcount) in other OSs than AIX
  *
@@ -75,27 +82,27 @@
  *
  * Revision 1.4  1996-09-17 21:43:14+02  fred
  * Moved under RCS, replace \n and \t with ?
- *
- *
  */
 
 static char *WhatString[]= {
-  "@(#)pstree $Revision: 2.16 $ by Fred Hucht (C) 1993-2000",
-  "@(#)EMail:fred@thp.Uni-Duisburg.DE",
-  "$Id: pstree.c,v 2.16 2000-03-01 10:42:22+01 fred Exp fred $"
+  "@(#)pstree $Revision: 2.17 $ by Fred Hucht (C) 1993-2002",
+  "@(#)EMail:fred@thp.Uni-Duisburg.de",
+  "$Id: pstree.c,v 2.17 2001-12-17 13:18:02+01 fred Exp $"
 };
 
 #define MAXLINE 512
 
 #if defined(_AIX) || defined(___AIX)	/* AIX >= 3.1 */
 /* Under AIX, we directly read the process table from the kernel */
+# ifndef _AIX50
+/* problems with getprocs() under AIX 5L
+ * workaround contributed by Chris Benesch <chris@fdbs.com> */
 #  define USE_GetProcessesDirect
-#  define HAS_PGID
+# endif /*_AIX50*/
 #  define HAS_TERMDEF
 extern char *termdef(int, char);
 #  define _ALL_SOURCE
 /*#  include <pwd.h>*/
-#  define NEED_UID2NAME
 #  include <procinfo.h>
 #  define USE_GETPROCS
 
@@ -112,8 +119,13 @@ extern getuser(struct procinfo *, int, void *, int);
 
 extern getargs(struct ProcInfo *, int, char *, int);
 
-#  define PSCMD 	"ps -ekf"
-#  define PSFORMAT 	"%s %ld %ld %*20c %*s %[^\n]"
+/*#  define PSCMD 	"ps -ekf"
+  #  define PSFORMAT 	"%s %ld %ld %*20c %*s %[^\n]"*/
+#  define HAS_PGID
+#  define UID2USER
+#  define PSCMD 	"ps -eko uid,pid,ppid,pgid,thcount,args"
+#  define PSFORMAT 	"%ld %ld %ld %ld %ld %[^\n]"
+#  define PSVARS	&P[i].uid, &P[i].pid, &P[i].ppid, &P[i].pgid, &P[i].thcount, P[i].cmd
 
 #elif defined(__linux)	/* Linux */
 #  define USE_GetProcessesDirect
@@ -121,8 +133,11 @@ extern getargs(struct ProcInfo *, int, char *, int);
 #  include <glob.h>
 #  include <sys/stat.h>
 #  define UID2USER
-#  define PSCMD 	"ps laxw"
-#  define PSFORMAT 	"%*d %ld %ld %ld %*37c %*s %[^\n]"
+/*#  define PSCMD 	"ps laxw"
+  #  define PSFORMAT 	"%*d %ld %ld %ld %*37c %*s %[^\n]"*/
+#  define PSCMD 	"ps -eo uid,pid,ppid,pgid,args"
+#  define PSFORMAT 	"%ld %ld %ld %ld %[^\n]"
+#  define PSVARS	&P[i].uid, &P[i].pid, &P[i].ppid, &P[i].pgid, P[i].cmd
 /* #elif defined(sparc)	SunOS */
 #elif defined(sun) && (!defined(__SVR4)) /* Solaris 1.x */
 /* contributed by L. Mark Larsen <mlarsen@ptdcs2.intel.com> */
@@ -165,8 +180,8 @@ extern getargs(struct ProcInfo *, int, char *, int);
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
 /* NetBSD contributed by Gary D. Duzan <gary@wheel.tiac.net> */
 #  define HAS_PGID
-#  define PSCMD               "ps -axwwo user,pid,ppid,pgid,command"
-#  define PSFORMAT    "%s %ld %ld %ld %[^\n]"
+#  define PSCMD 	"ps -axwwo user,pid,ppid,pgid,command"
+#  define PSFORMAT 	"%s %ld %ld %ld %[^\n]"
 #  define PSVARS	P[i].name, &P[i].pid, &P[i].ppid, &P[i].pgid, P[i].cmd
 #else			/* HP-UX, A/UX etc. */
 #  define PSCMD 	"ps -ef"
@@ -187,11 +202,6 @@ extern getargs(struct ProcInfo *, int, char *, int);
 #include <unistd.h>		/* For getopt() */
 #include <pwd.h>		/* For getpwnam() */
 
-#ifdef UID2USER
-/*#include <pwd.h>*/
-#define NEED_UID2NAME
-#endif
-
 #ifdef NEED_STRSTR
 static char *strstr(char *, char *);
 #endif
@@ -241,8 +251,8 @@ struct Proc {
   unsigned long thcount;
 } *P;
 
-#ifdef NEED_UID2NAME
-void uid2name(uid_t uid, char *name, int len) {
+#ifdef UID2USER
+void uid2user(uid_t uid, char *name, int len) {
 #define NUMUN 128
   static struct un_ {
     uid_t uid;
@@ -338,7 +348,7 @@ int GetProcessesDirect(void) {
     P[i].pgid    = proc[i].pi_pgrp;
     P[i].thcount = IFNEW(proc[i].pi_thcount, 1);
     
-    uid2name(P[i].uid, P[i].name, sizeof(P[i].name));
+    uid2user(P[i].uid, P[i].name, sizeof(P[i].name));
     
     if (IFNEW(proc[i].pi_state,proc[i].pi_stat) == SZOMB) {
       strcpy(P[i].cmd, "<defunct>");
@@ -427,7 +437,7 @@ int GetProcessesDirect(void) {
     if (k > 0) P[i].cmd[k] = '\0';
     fclose(tn);
     
-    uid2name(P[i].uid, P[i].name, sizeof(P[i].name));
+    uid2user(P[i].uid, P[i].name, sizeof(P[i].name));
     
 #ifdef DEBUG
     if (debug) fprintf(stderr,
@@ -523,7 +533,7 @@ int GetProcesses(void) {
     sscanf(line, PSFORMAT, PSVARS);
     
 #ifdef UID2USER	/* get username */
-    uid2name(P[i].uid, P[i].name, sizeof(P[i].name));
+    uid2user(P[i].uid, P[i].name, sizeof(P[i].name));
 #endif
 
 #ifdef DEBUG
@@ -662,7 +672,8 @@ void Usage(void) {
 #ifdef DEBUG
 	  "   -d        print debugging info to stderr\n"
 #endif
-	  "   -f file   read input from <file> (- is stdin) instead of running \"%s\"\n"
+	  "   -f file   read input from <file> (- is stdin) instead of running\n"
+	  "             \"%s\"\n"
 	  "   -g n      use graphics chars for tree. n=1: IBM-850, n=2: VT100\n"
 	  "   -u user   show only branches containing processes of <user>\n"
 	  "   -U        don't show branches containing only root processes\n"
@@ -761,7 +772,7 @@ int main(int argc, char **argv) {
   }
   
 #if defined(UID2USER) && defined(DEBUG)
-  if (debug) uid2name(0,NULL,0);
+  if (debug) uid2user(0,NULL,0);
 #endif
   MyPid = getpid();
   
@@ -784,9 +795,9 @@ int main(int argc, char **argv) {
   MarkProcs();
   DropProcs();
   
-  if (argc == optind) /* No pids */
+  if (argc == optind) { /* No pids */
     PrintTree(get_pid_index(1), "");
-  else while (optind < argc) {
+  } else while (optind < argc) {
     int idx;
     pid = (long)atoi(argv[optind]);
     idx = get_pid_index(pid);
